package java.nio.file.attribute;

import java.util.Iterator;
import java.util.Collections;
import java.util.HashSet;
import java.util.EnumSet;
import java.util.Set;

public final class PosixFilePermissions
{
    private PosixFilePermissions() {
    }
    
    private static void writeBits(StringBuilder sb, boolean b, boolean b2, boolean b3) {
        if (b) {
            sb.append('r');
        }
        else {
            sb.append('-');
        }
        if (b2) {
            sb.append('w');
        }
        else {
            sb.append('-');
        }
        if (b3) {
            sb.append('x');
        }
        else {
            sb.append('-');
        }
    }
    
    public static String toString(Set<PosixFilePermission> set) {
        StringBuilder sb = new StringBuilder(9);
        writeBits(sb, set.contains(PosixFilePermission.OWNER_READ), set.contains(PosixFilePermission.OWNER_WRITE), set.contains(PosixFilePermission.OWNER_EXECUTE));
        writeBits(sb, set.contains(PosixFilePermission.GROUP_READ), set.contains(PosixFilePermission.GROUP_WRITE), set.contains(PosixFilePermission.GROUP_EXECUTE));
        writeBits(sb, set.contains(PosixFilePermission.OTHERS_READ), set.contains(PosixFilePermission.OTHERS_WRITE), set.contains(PosixFilePermission.OTHERS_EXECUTE));
        return sb.toString();
    }
    
    private static boolean isSet(char c, char c2) {
        if (c == c2) {
            return true;
        }
        if (c == '-') {
            return false;
        }
        throw new IllegalArgumentException("Invalid mode");
    }
    
    private static boolean isR(char c) {
        return isSet(c, 'r');
    }
    
    private static boolean isW(char c) {
        return isSet(c, 'w');
    }
    
    private static boolean isX(char c) {
        return isSet(c, 'x');
    }
    
    public static Set<PosixFilePermission> fromString(String s) {
        if (s.length() != 9) {
            throw new IllegalArgumentException("Invalid mode");
        }
        EnumSet<PosixFilePermission> none = EnumSet.noneOf(PosixFilePermission.class);
        if (isR(s.charAt(0))) {
            none.add(PosixFilePermission.OWNER_READ);
        }
        if (isW(s.charAt(1))) {
            none.add(PosixFilePermission.OWNER_WRITE);
        }
        if (isX(s.charAt(2))) {
            none.add(PosixFilePermission.OWNER_EXECUTE);
        }
        if (isR(s.charAt(3))) {
            none.add(PosixFilePermission.GROUP_READ);
        }
        if (isW(s.charAt(4))) {
            none.add(PosixFilePermission.GROUP_WRITE);
        }
        if (isX(s.charAt(5))) {
            none.add(PosixFilePermission.GROUP_EXECUTE);
        }
        if (isR(s.charAt(6))) {
            none.add(PosixFilePermission.OTHERS_READ);
        }
        if (isW(s.charAt(7))) {
            none.add(PosixFilePermission.OTHERS_WRITE);
        }
        if (isX(s.charAt(8))) {
            none.add(PosixFilePermission.OTHERS_EXECUTE);
            return none;
        }
        return none;
    }
    
    public static FileAttribute<Set<PosixFilePermission>> asFileAttribute(Set<PosixFilePermission> c) {
        final HashSet set = new HashSet(c);
        Iterator<Object> iterator = set.iterator();
        while (iterator.hasNext()) {
            if ((PosixFilePermission)iterator.next() == null) {
                throw new NullPointerException();
            }
            continue;
        }
        return new FileAttribute<Set<PosixFilePermission>>() {
            @Override
            public String name() {
                return "posix:permissions";
            }
            
            @Override
            public Set<PosixFilePermission> value() {
                return Collections.unmodifiableSet(set);
            }
        };
    }
}
